@@ -331,6 +331,10 @@ configuration.
         self.dyn_running = prev_dp.dyn_running
         self.dyn_up_port_nos = set(prev_dp.dyn_up_port_nos)
         self.dyn_last_coldstart_time = prev_dp.dyn_last_coldstart_time
+        if self.stack:
+            self.stack.clone_dyn_state(prev_dp.stack)
+        for number in self.ports:
+            self.ports[number].clone_dyn_state(prev_dp.ports.get(number))
 
     def cold_start(self, now):
         """Update to reflect a cold start"""
@@ -1374,16 +1378,6 @@ configuration.
                     changed_acl_ports.add(port_no)
                     logger.info('port %s ACL changed (ACL %s to %s)' % (
                         port_no, old_acl_ids, new_acl_ids))
-            # If any of the changed VLANs have tagged ports we will need to restore their flows,
-            # even if those tagged ports didn't change, as the delete VLAN operation uses
-            # a wildcard delete.
-            for port_no in same_ports:
-                old_port = self.ports[port_no]
-                new_port = new_dp.ports[port_no]
-                for tagged_vlan in new_port.tagged_vlans:
-                    if tagged_vlan.vid in changed_vlans:
-                        logger.info('port %s has a changed tagged VLAN' % port_no)
-                        changed_ports.add(port_no)
 
             if changed_acl_ports:
                 same_ports -= changed_acl_ports
